#include <raylib.h>
#include "preferred_size.h"
#include <math.h>
#include <raymath.h>

int main()
{
  int screenWidth, screenHeight;
  GetPreferredSize(&screenWidth, &screenHeight);
  InitWindow(screenWidth, screenHeight, "Billboard rendering");
  SetTargetFPS(30);

  // Define the camera to look into our 3d world
  Camera camera = {0};
  camera.position = (Vector3){5.0f, 8.0f, 5.0f}; // Camera position
  camera.target = (Vector3){0.0f, 0.5f, 0.0f};   // Camera looking at point
  camera.up = (Vector3){0.0f, 1.0f, 0.0f};       // Camera up vector (rotation towards target)
  camera.fovy = 45.0f;                           // Camera field-of-view Y
  camera.projection = CAMERA_PERSPECTIVE;        // Camera projection type

  Texture2D bill = LoadTexture("data/sprite.png");
  Vector3 billPositionStatic = {0.0f, 2.0f, 0.0f};

  while (!WindowShouldClose())
  {
    if (IsPaused())
    {
      // canvas is not visible in browser - do nothing
      continue;
    }
    BeginDrawing();
    ClearBackground(DARKGREEN);
    camera.position.x = sinf(GetTime() * 0.25f) * 5.0f;
    if (IsMouseButtonDown(MOUSE_LEFT_BUTTON))
    {
      camera.position.y += GetMouseDelta().y * 0.05f;
    }
    
    BeginMode3D(camera);
    DrawGrid(4, 1.0f);
    DrawCube((Vector3){0.0f, 0.0f, 0.0f}, 0.25f, 0.25f, 0.25f, BLUE);

    Rectangle source = { 0.0f, 0.0f, (float)bill.width, (float)bill.height };

    Vector3 forward = Vector3Subtract(camera.target, camera.position);
    Vector3 up = { 0.0f, 1.0f, 0.0f };
    Vector3 right = Vector3CrossProduct(up, forward);
    up = Vector3CrossProduct(forward, right);
    up = Vector3Normalize(up);
    float scale = 2.0f;
    Vector2 size = (Vector2) { scale*fabsf((float)source.width/source.height), scale };

    DrawLine3D((Vector3){0.0f, 0.0f, 0.0f}, Vector3Scale(forward, 2.0f), RED);
    DrawLine3D((Vector3){0.0f, 0.0f, 0.0f}, Vector3Scale(up, 2.0f), GREEN);
    DrawLine3D((Vector3){0.0f, 0.0f, 0.0f}, Vector3Scale(right, 2.0f), BLUE);

    for (float z = -10.0f; z <= 10.0f; z += 2.0f)
    {
      Vector3 billPosition = billPositionStatic;
      billPosition.z = z;
      billPosition.x = 2.0f;
      // a little hopping
      billPosition.y = fabsf(sinf(GetTime() * 5.0f + z) * 0.25f);
      DrawBillboard(camera, bill, billPosition, scale, WHITE);

      billPosition.x = -2.0f;
      DrawBillboardPro(camera, bill, source, billPosition, up, size, Vector2Scale(size, 0.5), 0.0f, WHITE);
    }
    EndMode3D();
    EndDrawing();
  }

  CloseWindow();

  return 0;
}